<?php

namespace Drupal\Tests\userlike\Functional;

use Drupal\Tests\BrowserTestBase;

// Include userlike constants file:
use Drupal\userlike\Helper\Constants;

/**
 * This class provides methods for testing the userlike browser functionalities.
 *
 * @group userlike
 */
class UserlikeFunctionalTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'userlike',
    'node',
    'test_page_test',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * An admin user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminuser;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The userlike dummy account key for testing.
   *
   * @var string
   */
  protected $userlikeAccountKey = 'xxxxxx';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Use the test page as the front page.
    $this->config('system.site')->set('page.front', '/test-page')->save();
    // Create an user:
    $this->user = $this->drupalCreateUser([
      'administer userlike',
      'opt-in or out of userlike tracking',
      'bypass node access',
      'access content overview',
    ]);
    // Create an admin user:
    $this->adminuser = $this->drupalCreateUser(['administer userlike']);
    $this->adminuser->addRole($this->createAdminRole('administrator', 'administrator'));
    $this->adminuser->save();
    // Login as regular authenticated user by default:
    $this->drupalLogin($this->user);
  }

  /**
   * Tests if the form is unaccessible as an anonymous user.
   */
  public function testUserlikeAccessFormAsAnonymous() {
    $this->drupalLogout();
    $session = $this->assertSession();
    // Go to settings and see if they exist:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(403);
  }

  /**
   * Tests if the form is accessible as an admin user.
   */
  public function testUserlikeAccessFormAsAdmin() {
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    $session = $this->assertSession();
    // Go to settings and see if they exist:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
  }

  /**
   * Tests saving the account key in form to configuration.
   */
  public function testUserlikeFormAccountKey() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to settings and see if they exist:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Fill the account key:
    $page->fillField('edit-account-key', $this->userlikeAccountKey);
    $page->pressButton('edit-submit');
    $session->statusCodeEquals(200);
    $session->pageTextContains('The configuration options have been saved.');
    // Check if the account key is set in the config:
    $this->assertEquals($this->userlikeAccountKey, \Drupal::config('userlike.settings')->get('userlike_secret'));
  }

  /**
   * Tests if the userlike script exists in the header.
   */
  public function testUserlikeInHeaderExists() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    // See if account key is the same as in the config:
    $this->assertEquals($this->userlikeAccountKey, $this->config('userlike.settings')->get('userlike_secret'));
    $this->config('userlike.settings')->set('userlike_script_position', 'header')->save();
    $this->drupalGet('<front>');
    // Check parent is head:
    $this->assertSession()->elementExists('css', 'head script#_etLoader');
    $this->assertSession()->elementExists('css', 'head script#userlike_script');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('<front>');
    // Check parent is head:
    $this->assertSession()->elementExists('css', 'head script#_etLoader');
    $this->assertSession()->elementExists('css', 'head script#userlike_script');
  }

  /**
   * Tests if the userlike script exists in the footer.
   */
  public function testUserlikeInFooterExists() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $this->config('userlike.settings')->set('userlike_script_position', 'footer')->save();
    $this->drupalGet('<front>');
    // Only check if parent element is body:
    $this->assertSession()->elementExists('css', 'body script#_etLoader');
    $this->assertSession()->elementExists('css', 'body script#userlike_script');
    // By default, anonymous users should see the same script.
    $this->drupalLogout();
    $this->drupalGet('<front>');
    // Check parent is body:
    $this->assertSession()->elementExists('css', 'body script#_etLoader');
    $this->assertSession()->elementExists('css', 'body script#userlike_script');
  }

  /**
   * Tests if cookie-less tracking is enabled.
   */
  public function testDataBlockCookieValueEnabled() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $this->drupalGet('<front>');
    // Go to front page and see if the value is set in the script:
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementAttributeContains('css', 'script#_etLoader', 'data-block-cookies', 'true');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementAttributeContains('css', 'script#_etLoader', 'data-block-cookies', 'true');
  }

  /**
   * Tests if Do-Not_track is enabled.
   */
  public function testDataRespectDntValueEnabled() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    // Go to front page and see if the value is set in the script:
    $this->drupalGet('<front>');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementAttributeContains('css', 'script#_etLoader', 'data-respect-dnt', 'true');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementAttributeContains('css', 'script#_etLoader', 'data-respect-dnt', 'true');
  }

  /**
   * Tests if the script is not loaded, including a specific path with a slash.
   */
  public function testExclusionByPathWithSlashUrl() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Create node1:
    $this->createNode(['type' => 'article', 'id' => 1, 'title' => 'test123']);
    // Don't track on node1:
    $page->fillField('edit-userlike-track-path-mode-all-pages', 'all_pages');
    $page->fillField('edit-userlike-track-paths', '/node/add');
    $page->pressButton('edit-submit');
    // See if script won't load for our authenticated user:
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded, including a specific path with a slash.
   */
  public function testInclusionByPathWithSlashUrl() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Create node1:
    $this->createNode(['type' => 'article', 'id' => 1, 'title' => 'test123']);
    // Don't track on node1:
    $page->fillField('edit-userlike-track-path-mode-all-pages', 'listed_pages');
    $page->fillField('edit-userlike-track-paths', '/node/add');
    $page->pressButton('edit-submit');

    // See if script won't load for our authenticated user:
    $this->drupalGet('/node/add');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('/node/add');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded, including a specific path without slash.
   */
  public function testExclusionByPathWithoutSlashUrl() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Create node1:
    $this->createNode(['type' => 'article', 'id' => 1, 'title' => 'test123']);
    // Don't track on node1:
    $page->fillField('edit-userlike-track-path-mode-all-pages', 'all_pages');
    $page->fillField('edit-userlike-track-paths', 'node/add');
    $page->pressButton('edit-submit');

    // See if script won't load for our authenticated user:
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded, including a specific path without a slash.
   */
  public function testInclusionByPathWithoutSlashUrl() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Create node1:
    $this->createNode(['type' => 'article', 'id' => 1, 'title' => 'test123']);
    // Don't track on node1:
    $page->fillField('edit-userlike-track-path-mode-all-pages', 'listed_pages');
    $page->fillField('edit-userlike-track-paths', 'node/add');
    $page->pressButton('edit-submit');

    // See if script won't load for our authenticated user:
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // By default, anonymous users should have the same result.
    $this->drupalLogout();
    $this->drupalGet('/node/add');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an authenticated user.
   */
  public function testExclusionByRoleAuthenticatedTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except authenticated user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->pressButton('edit-submit');
    // See if script won't load for our authenticated user:
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // Anonymous users should still "see" the script:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an authenticated user.
   */
  public function testInclusionByRoleAuthenticatedTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track authenticated user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->pressButton('edit-submit');
    // See if script will load for our authenticated user:
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
    // Anonymous users shouldn't see the script:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an authenticated user.
   */
  public function testExclusionByRoleAuthenticatedMultipleTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except authenticated user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // See if script won't load for our authenticated user:
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // Anonymous users also shouldn't see the script:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an authenticated user.
   */
  public function testInclusionByRoleAuthenticatedMultipleTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track authenticated user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // See if script will load for our authenticated user:
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
    // Anonymous users also should see the script:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an anonymous user.
   */
  public function testExclusionByRoleAnonymousTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except anonymous user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // Logout and see if script won't load for a an anonymous user:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // Authenticated users should still see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an anonymous user.
   */
  public function testInclusionByRoleAnonymousTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track anonymous user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // Logout and see if script will load for an anonymous user:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
    // Authenticated users shouldn't see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an anonymous user.
   */
  public function testExclusionByRoleAnonymousMultipleTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except anonymous user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // Logout and see if script won't load for a an anonymous user:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
    // Authenticated users also shouldn't see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an anonymous user.
   */
  public function testInclusionByRoleAnonymousMultipleTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track anonymous user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    // Logout and see if script will load for an anonymous user:
    $this->drupalLogout();
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
    // Authenticated users should also see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an admin user.
   */
  public function testExclusionByRoleAdminTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except admin user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-administrator', 'administrator');
    $page->pressButton('edit-submit');
    $this->drupalLogout();
    // Check if anonymous users still see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Check if authenticated users still see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Login as adminuser:
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    // Admin user shouldn't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an admin user.
   */
  public function testInclusionByRoleAdminTest() {
    $this->drupalLogout();
    // Login as adminuser:
    $this->drupalLogin($this->adminuser);
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track admin user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-administrator', 'administrator');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    $this->drupalLogout();
    // Check if anonymous users still see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Check if authenticated users still see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');

    // Login as adminuser:
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    // Admin user shouldn't see the script:
    $this->drupalGet('<front>');
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is not loaded on an admin user.
   */
  public function testExclusionByRoleAdminMultipleTest() {
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Track all roles except admin user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'all_roles');
    $page->fillField('edit-userlike-track-roles-administrator', 'administrator');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    $this->drupalLogout();
    // Check if anonymous users don't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');

    // Check if authenticated users still see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Login as adminuser:
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    // Admin user shouldn't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an admin user.
   */
  public function testInclusionByRoleAdminMultipleTest() {
    $this->drupalLogout();
    // Login as adminuser:
    $this->drupalLogin($this->adminuser);
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track admin user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-administrator', 'administrator');
    $page->fillField('edit-userlike-track-roles-anonymous', 'anonymous');
    $page->pressButton('edit-submit');
    $this->drupalLogout();
    // Anonymous users should see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Authenticated users shouldn't see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');

    // Login as adminuser:
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    // Admin user shouldn't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

  /**
   * Tests if the script is loaded on an admin user, inherited from authenticated.
   */
  public function testInclusionByRoleAuthenticatedInheritTest() {
    $this->drupalLogout();
    // Login as adminuser:
    $this->drupalLogin($this->adminuser);
    // Set account key:
    $this->config('userlike.settings')->set('userlike_secret', $this->userlikeAccountKey)->save();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to userlike settings page:
    $this->drupalGet('/admin/config/system/userlike');
    $session->statusCodeEquals(200);
    // Only track admin user:
    $page->fillField('edit-userlike-track-roles-mode-all-roles', 'listed_roles');
    $page->fillField('edit-userlike-track-roles-administrator', 'administrator');
    $page->fillField('edit-userlike-track-roles-authenticated', 'authenticated');
    $page->pressButton('edit-submit');
    $this->drupalLogout();
    // Anonymous users shouldn't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementNotExists('css', 'script#_etLoader');
    $this->assertSession()->elementNotExists('css', 'script#userlike_script');

    // Authenticated users still see the script:
    $this->drupalLogin($this->user);
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');

    // Login as adminuser:
    $this->drupalLogout();
    $this->drupalLogin($this->adminuser);
    // Admin user shouldn't see the script:
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
    $this->assertSession()->elementExists('css', 'script#_etLoader');
    $this->assertSession()->elementExists('css', 'script#userlike_script');
  }

}
